;BREADAMP v05 - a sample music player for the Commodore 64, coded in 2020 by daddlerTL
;-------------------------------------------------------------------------------------
;
;This program plays sampled music from BREADAMP files loaded into the REU of a 1541ultimate or Turbo Chameleon 64.
;
;The BREADAMP files can be created from wav files using the BREADAMP_v05_REUmaker.
;
;The first version of BReadAMP was based on the video player BR-TV from Bamse/squidward which had two disadvantages:
;no controls and the sound quality was low because the video player was specialized in playing video, not audio only.
;
;All versions from v02 are standalone versions. They provide a better audio quality and the following controls: 
;track change, fast forward / rewind, pause / play and infobar controls. v03 allows to change the order in which the
;tracks are played, the change of border, background and text color and a "disco mode" that changes the border and/or
;background color every second of playback and some more (see readme.txt for further information).
;v04 includes the FPGASID recorder that allows audio sampling using the audio in, a title name editor, a faster player engine
;and a track repeat function.
;v05 allows the playback using the Ultimate Audio of the Ultimate 64 or 1541ultimate2, the playback using Digimax is also supported
;The later versions contain all function of the previous versions and are backwards compatible (all files for v02 or higher can be played).
;
;This version supports the following audio formats:
;PAL: 15638 / 11728 / 7819 / 5864 Hz mono and 11728 / 7819 / 5864 Hz stereo using the "Mahoney" format and 7819 / 5864 Hz mono 4 Bit
;NTSC: 15734 / 7867 Hz mono and 7867 Hz stereo using the "Mahoney" format and 7867 Hz Mono 4 Bit 
;(the number of supported formats is limited for NTSC due to disadvantageos distribution of VIC "bad lines" / uneven number of lines)
;Ultimate Audio supports those formats: 48000 / 44100 / 32000 / 22050 / 15734 / 15638 / 11728 / 7867 / 7819 / 5864 Hz 
;mono / stereo 8 / 16 Bit on both PAL and NTSC machines
;
;This is my first assembler project so the coding style might not meet the demoscene standards, but that was not my goal,
;my goal was to create a working music player to be able to listen to "normal / non-SID" music using my C64.
;
;The player and this assembler code is freeware. If you publish an altered version of the player please provide the altered
;source code and keep my name daddlerTL in the file.
;
;The source code was written using C64Studio and can be compiled using this program
;
;Additional to this file the following files are required to be able to compile the program:
;-Breadampv05_REUFS_Parser.asm: contains the routines to interpret the contents of the REU file (tracknames, formats, meta data etc.)
;-Breadampv05_Tastaturfunktionen.asm: contains the functions you can access using the keyboard/joystick(s) (track change, pause etc.)
;-Breadampv05_Unterroutinen_und_Daten.asm: contains sub routines, lookup tables and texts (error messages, text of the infobar etc.)
;-Breadampv05_FPGASID_recorder: contains the routines of the FPGASID recorder
;-charset.bin: the charset used by the player
;-screen.bin: the initial content of the screen (player layout) which is loaded during program startup
;All files have to be in the same directory as this main file.
;
;--------------------------------------------------------------------------------------------------------------------------------------
;
;BREADAMP v05 - ein Samplemusikplayer für den Commodore 64, programmiert 2020 von daddlerTL
;------------------------------------------------------------------------------------------
;
;Dieses Programm spielt gesamplete Musik von BREADAMP-Dateien aus der REU einer 1541ultimate oder eines Turbo Chameleon 64 ab.
;
;Die BREADAMP-Dateien koennen aus wav-Dateien mit dem Programm BREADAMP_v05_REUmaker erstellt werden. 
;
;Die erste Version von BReadAMP basierte auf dem Videoplayer BR-TV von Bamse/squidward, was zwei Nachteile hatte:
;keine Steuerungsmoeglichkeiten und schlechte Tonqualität, weil der Videoplayer nicht auf reine Tonwiedergabe optimiert war.
;
;Alle Versionen ab v02 sind Standalone Versionen. Sie bieten eine bessere Tonqualität und folgende Steuerungsmoeglichkeiten:
;Titelwechsel, Vor- und Zurueckspulen, Pause / Play und Steuerung der Infoleiste. v03 ermöglicht die Aenderung der Titelreihenfolge,
;das Aendern der Rahmen-, Hintergrund- und Textfarbe und einen "Diskomodus", der die Rahmen- und/oder Hintergrundfarbe jede Sekunde
;aendert und einiges mehr (siehe readme.txt fuer weitere Informationen).
;v04 beinhaltet den FPGASID-Rekorder, der das Sampling von Ton mit Hilfe des Toneingangs ermoeglicht, einen Titelnameneditor, eine schnellere
;Playerengine und eine Titelwiederholfunktion.
;v05 erlaubt die Wiedergabe mittels Ultimate Audio des Ultimate 64 oder der 1541ultimate2, das DigiMax wird ebenfalls unterstuetzt.
;Die neueren Versionen beinhalten alle Funktionen der vorherigen und sind abwaertskompatibel (alle Dateien fuer v02 und hoeher abspielbar).
;
;Diese Version unterstützt die folgenden Tonformate:
;PAL: 15638 / 11728 / 7819 / 5864 Hz Mono und 11728 / 7819 / 5864 Hz Stereo im "Mahoney"-Format und 7819 / 5864 Hz Mono 4 Bit
;NTSC: 15734 / 7867 Hz Mono und 7867 Hz Stereo im "Mahoney"-Format und 7867 Hz Mono 4 Bit 
;(die Anzahl der unterstützen Formate ist begrenzt bei NTSC wegen der ungünstigen Verteilung der VIC "bad lines" / ungeraden Zeilenanzahl)
;Ultimate Audio supports those formats: 48000 / 44100 / 32000 / 22050 / 15734 / 15638 / 11728 / 7867 / 7819 / 5864 Hz 
;mono / stereo, 8 / 16 Bit sowohl auf PAL- also auch auf NTSC-Geraeten
;
;Dies ist mein erstes Assemblerprojekt, der Programmierstil wird wohl nicht den Standards der Demoszene entsprechen, aber das
;war auch nicht mein Ziel, sondern mein Ziel war die Erstellung eines funktionierenden Musikabspielprogramms, mit dem ich
;"normale / Nicht-SID"-Musik mit meinem C64 hören kann.
;
;Der Player und dieser Assembler-Code sind Freeware. Wenn du eine geaenderte Version des Players veroeffentlichst, bitte den geaenderten
;Quellcode beilegen und meinen Name daddlerTL in der Datei lassen.
;
;Der Quellcode wurde mit dem Programm C64Studio geschrieben und kann mit diesem kompiliert werden.
;
;Neben dieser Datei werden noch folgende Dateien benötigt, um das Programm kompilieren zu können:
;-Breadampv05_REUFS_Parser.asm: enthaelt die Routinen zur Interpretation des Inhalts der REU-Datei (Titelnamen, Formate, Metadaten usw.)
;-Breadampv05_Tastaturfunktionen.asm: enthaelt die Funktionen, die man mittels Tastatur/Joystick(s) aufrufen kann  (Trackwechsel, Pause usw.)
;-Breadampv05_Unterroutinen_und_Daten.asm: enthaelt Unterroutinen, Lookuptabellen und Texte (Fehlermeldungen, Text fuer die Infoleiste usw.)
;-Breadampv05_FPGASID_recorder: enthaelt die Routinen des FPGASID-Rekorders
;-charset.bin: der vom Player benutzte Zeichensatz
;-screen.bin: der initale Bildschirminhalt (Playerlayout) der beim Programmstart geladen wird
;Alle Dateien muessen sich im gleichen Verzeichnis befinden wie diese Hauptdatei.


;*** Variablen/Adressen
REU_CMD      = $DF01      ;REU-Register
REU_C64Lo    = $DF02
REU_C64Hi    = $DF03
REU_Lo       = $DF04
REU_Hi       = $DF05
REU_Bank     = $DF06
REU_BytesLo  = $DF07
REU_BytesHi  = $DF08 
REU_Adr_Ctrl = $DF0A

UA           = $DF20      ;Startadresse fuer Ultimate Audio API

header       = $C000      ;Die ersten $300 Bytes aus dem REU-Files (zwecks Parsing kopiert)
trackstart_h = $C300      ;Startadresse eines Tracks in der REU (3 Bytes pro Eintrag)              
trackstart_m = $C310      
trackstart_l = $C320      
tracklaenge_h= $C330      ;Laenge eines Tracks in Bytes (3 Bytes pro Eintrag)                        
tracklaenge_m= $C340                            
tracklaenge_l= $C350                            
trackende_h  = $C360      ;Endadresse eines Tracks in der REU (3 Bytes pro Eintrag), wird berechnet   
trackende_m  = $C370       
trackende_l  = $C380        
trackmin_s_z = $C390      ;Spielzeit Minuten  (Screencode Zehner), wird berechnet  
trackmin_s_e = $C3A0      ;Spielzeit Minuten  (Screencode Einer), wird berechnet  
tracksek_h   = $C3B0      ;Spielzeit Sekunden, wird berechnet 
tracksek_l   = $C3C0      
tracksek_s_z = $C3D0      ;Spielzeit Sekunden (Screencode Zehner), wird berechnet  
tracksek_s_e = $C3E0      ;Spielzeit Sekunden (Screencode Einer), wird berechnet 
trackinfos_h = $C3F0      ;Startadresse der Trackinfos in der REU (ohne Banknummer, da immer 0)
trackinfos_l = $C3F1

tracknr_z    = $C400      ;Tracknummer Zehnerstelle
tracknr_e    = $C410      ;Tracknummer Einerstelle
track_tvnorm = $C420      ;Norm (0 = PAL, 1 = NTSC)
track_format = $C430      ;Format (000 = unbekannt (4bit oder Datei v02), 001 = 6581, 010 = 8580, 011 = FPGASID, 100 = Digimax, 101 = 6581+8580 (nur Stereo), 110 = 8580+6581 (nur Stereo), 111 = Ultimate Audio)
track_sidtyp = $C440      ;SID-Typ, fuer den die Datei erstellt wurde: 000=unbekannt, 001=6581, 010=8580, 011=FPGASID, 101=6581+8580 (nur Stereo), 110=8580+6581 (nur Stereo)
track_stereo = $C450      ;1 = Stereo, 0 = Mono
track_bits   = $C460      ;Bits pro Sample (4 oder 8)
track_srate  = $C470      ;Samplerate gerundet (Bit 7-4 x1 kHz, Bit 3-0 x100 Hz)
track_srate_z= $C480      ;Samplerate (Zehnerstelle) fuer die Anzeige
track_srate_e= $C490      ;Samplerate (Einerstelle) fuer die Anzeige
track_takte  = $C4A0      ;Takte zwischen den Samples (legt Abspielgeschwindigkeit fest)
lup_takte    = $C4B0      ;Lookuptable fuer Takte zwischen den Samples (legt Abspielgeschwindigkeit fest)
lup_srate    = $C4B8      ;Lookuptable fuer Samplerate
lup_sr_zehner= $C4C0      ;Lookuptable fuer Screencodes der Samplerate (Zehner)
lup_sr_einer = $C4C8      ;Lookuptable fuer Screencodes der Samplerate (einer)
lup_sr_m     = $C4D0      ;Lookuptable fuer Spielzeitberechnungen (Mid-Byte, Highbyte nicht in Lookuptable)
lup_sr_l     = $C4D8      ;Lookuptable fuer Spielzeitberechnungen (Low-Byte)
lup_sr_st_m  = $C4E0      ;Lookuptable fuer Spielzeitberechnungen (High-Byte, Stereo, Highbyte nicht in Lookuptable)
lup_sr_st_l  = $C4E8      ;Lookuptable fuer Spielzeitberechnungen (Low-Byte, Stereo)
lup_sr_4b_h  = $C4F0      ;Lookuptable fuer Spielzeitberechnungen (High-Byte, 4 Bit)
lup_sr_4b_l  = $C4F4      ;Lookuptable fuer Spielzeitberechnungen (Low-Byte, 4 Bit)


dividend     = $C500      ;Hilfsvariablen fuer Division 
divisor      = $C503      ;(je 3 Bytes)
maxtrack     = $C506      ;hoechste Tracknummer    (1 Byte)
taste        = $C507      ;Flag, ob eine Taste gedrueckt ist (zur Vermeidung von unerwuenschter Mehrfachausfuehrung von Befehlen)
shift        = $C508      ;gibt an, ob eine Shift-Taste gedrueckt ist (0 = nein, 1 = linke, 2 = rechte Shift-Taste)
commodore    = $C509      ;Flag, ob die Commodore-Taste gedrueckt ist
eingabemodus = $C50A      ;Eingabemodus: 0 = Player-GUI, 1 = Titelnameneditor
gedr_taste   = $C50B      ;gedrueckte Taste (wird im Titelnameneditor benutzt)
fpgasid_datei= $C50C      ;gibt an, ob die Datei fuer den FPGASID erstellt wurde (0 = nein, 1 = ja)
fpgasid_vorh = $C50D      ;gibt an, ob ein FPGASID im C64 vorhanden ist
sidmanuell   = $C50E      ;manuelle Angabe des SIDs fuer FPGASID-Konfiguration fuer Musikdateien der v02 (0=6581, 1=8580)
statusanzeige= $C50F      ;Einstellung fuer Formatanzeige (0 = aus, 1 = kurz an (standard), 2 = immer an)
status_zeit  = $C510      ;Restzeit fuer Formatanzeige (wird von der Spielzeitanzeige runtergezaehlt, wenn 0 dann wird Anzeige abgeschaltet)
status_sek   = $C511      ;Sekunden, die zuletzt von der Spielzeitanzeige angezeigt wurden (wenn sie wechseln, wird status_zeit dekrementiert)
tvnorm_c64   = $C512      ;erkannte TV-Norm des C64 (0 = PAL, 1 = NTSC)
disco_rahmen = $C513      ;Discomodus fuer den Rahmen (0 = aus, 1 = an)
disco_hinterg= $C514      ;Discomodus fuer den Hintergrund (0 = aus, 1 = an)
fpgasid_fehlt= $C515      ;Flag fuer Fehler, dass Tracks fuer FPGASID in Datei, aber kein FPGASID vorhanden
tvnorm_falsch= $C516      ;Flag fuer Fehler, dass die TV-Norm des C64 nicht mit der der Datei uebereinstimmt (zur Vermeidung der Wiederholung der Fehlermeldung)
cursor_zeit  = $C517      ;Restzeit fuer Trackcursoranzeige (wird von der Spielzeitanzeige runtergezaehlt, wenn 0 dann wird Anzeige abgeschaltet)
cursor_pos   = $C518      ;Position des Trackcursors
hintergfarbe = $C519      ;Hintergrundfarbe (kann mit F3/F4 geaendert werden)
textfarbe    = $C51A      ;Textfarbe (kann mit F5/F6 geaendert werden)
steuerung    = $C51B      ;legt fest, wie die Steuerung erfolgt: 0 = Tastatur + Joystick Port 2, 1 = Joystick Port 1+2
feuer1serie  = $C51C      ;zaehlt, wie oft hintereinander Feuer an Joy Port 1 gedrueckt wurde (zaehlt bis 2 und fuehrt dann Textfarbenwechsel aus - beim 3. mal)
joy1feuer    = $C51D      ;1, wenn letzte Joystickaktion Port 1 Feuer war
starttrack   = $C51E      ;zuerst zu spielender Track (normalerweise 0, nach Aufnahme der aktuell aufgenommene)

infobar      = $C520      ;Infoleiste 0=deaktiviert / 1=aktiviert
infozeit     = $C521      ;Zeitzaehler fuer Infoleiste (zwecks Geschwindigkeitsbegrenzung)
infospeed    = $C522      ;Scrollgeschwindigkeit der Infoleiste

te_crsrpos   = $C530      ;Titelnameneditor: Cursorposition
te_cursor    = $C531      ;Cursor 0 = Titelchar (z.B. Buchstabe), 1 = Cursorchar
te_char      = $C532      ;Charbuffer für den Titelnameneditor
te_timer     = $C533      ;Timerzaehler fuer den Cursor
te_zeile     = $C534      ;zu bearbeitende Zeile
te_zstart_h  = $C535      ;Zeilenbeginn (abhängig vom gewaehlten Track)
te_zstart_l  = $C536 
te_taste     = $C537      ;letzte gueltige Tastenkombination (fuer Wiederholsperre)
te_backup    = $C538      ;Backup des Titelnamens (wird bei Abbruch wiederhergestellt) und der REU-Parameter, 29 ($1D) Bytes 
te_blinkzeit = $C555      ;Blinkzeit des Cursors (wird vom Programm variiert)

rec_start_h  = $C600      ;Adresse, an der der FPGASID-Recorder mit der Aufnahme beginnen kann (wird aus den Metadaten in der REU berechnet)
rec_start_m  = $C601
rec_start_l  = $C602
rec_srate    = $C603      ;Samplerate, mit der aufgenommen werden soll (Index 0...3)
rec_ueberlauf= $C604      ;Flag fuer REU-Ueberlauf, Aufnahme wird sofort beendet
rec_titeled  = $C605      ;Flag fuer Nutzung des Titelnameneditors - wurde er während der Aufnahme nicht benutzt, wird ein Standardname vergeben
rec_aufnahme = $C606      ;1 = Aufnahme laeuft, sonst 0

ua_track_mult= $C610      ;Multiplikator bei Ultimate Audio Tracks (zur Positionsbestimmung / Spielzeitberechnung), 14 Bytes
ua_track_T_h = $C620      ;Samplerateteiler High-Byte (14 Bytes) 
ua_track_T_l = $C630      ;Samplerateteiler Low-Byte (14 Bytes) 
ua_versatz_st= $C640      ;Adressenversatz bei Ultimate Audio Stereo (1 bei 8 Bit-Tracks, 2 bei 16 Bit-Tracks)
ua_spulen_f1 = $C650      ;Spulfaktur fuer die Reserve (14 Bytes)
ua_spulen_f2 = $C660      ;Spulfaktor fuer die Adresserhoehung (14 Bytes)
ua_vorh      = $C66E      ;Ergebnis der Pruefung, ob Ultimate Audio aktiviert ist (0 = definitiv nein, 1 = ja)
ua_reparse   = $C66F      ;Flag fuer Neuparsen der REU-Daten (bei Umschaltung zwischen Ultimade Audio und FPGASID)
ua_hq        = $C670      ;Flag fuer High Quality Ultimate Audio Tracks (unterbindet Umschalten zu FPGASID)

digimax_adr_h= $C680      ;Digimax-Adresse (wird waehrend des REU-Parsings ermittelt und kann im laufenden Betrieb umgeschaltet werden)
digimax_adr_l= $C681      ;Userport: $DD01, Cartridge: $DE00

track_takte_u= $C690      ;Hilfstabelle fuer die Umschaltung zwischen Ultimate Audio und FPGASID (Anzahl der Takte zwischen den Samples)
track_takte_f= $C6A0

                          ;da das Programm Kernal und Basic abschaltet, sind alle Zeropageadressen von $02 bis $FF frei
                          
track_sr_h     = $02        ;Samplerate der Tracks (fuer Berechnungen, High-Byte) 14 Bytes
track_sr_m     = $10        ;Samplerate der Tracks (fuer Berechnungen, Mid-Byte) 14 Bytes
track_sr_l     = $1E        ;Samplerate der Tracks (fuer Berechnungen, Low-Byte)  14 Bytes
track_player_h = $2C        ;Zeiger auf zu startenden Player (High-Byte) 14 Bytes 
track_player_l = $3A        ;Zeiger auf zu startenden Player (Low-Byte)  14 Bytes
track_init_h   = $49        ;Zeiger auf zu nutzende Playerinitialisierung (High-Byte) 14 Bytes 
track_init_l   = $57        ;Zeiger auf zu nutzende Playerinitialisierung (Low-Byte)  14 Bytes
halbbyte       = $65        ;Zeiger auf das abzuspielende Halbbyte bei 4Bit (0=linkes, 1=rechtes Halbbyte)
sample         = $66        ;Samplepuffer fuer 4Bit-Wiedergabe
track_init_a_h = $67        ;aktueller Trackinit (wird genutzt, um zu prüfen, ob beim Trackwechsel
track_init_a_l = $68        ;eine erneute Playerinitialisierung nötig ist)
sidtyp_a       = $69        ;aktueller SID-Typ (wird genutzt, um zu prüfen, ob beim Trackwechsel eine erneute Playerinitialisierung nötig ist)
rec_maxpegel   = $6A        ;Recorder: maximaler Pegel (dient der "Aussteuerungsanzeige")
trackrepeat    = $6B        ;Trackrepeatfunktion (0 = aus, 1 = an)
akt_takte      = $6C        ;aktuelle Takte zw. Samples (Wiedergabegeschwindigkeit)

altern_playlist= $70        ;Alternativplayliste (wird angepasst, wenn man die Tracks in der Reihenfolge verschiebt) 14 Bytes

adr1_l         = $8B        
adr1_h         = $8C
adr2_l         = $8D
adr2_h         = $8E
wechsel        = $8F        ;Wechselschalter: zur Lastreduzierung wird abwechselnd die Spielzeit und der Fortschrittsbalken berechnet

track          = $A6        ;aktuell gespielter Track
naechst_track  = $A7        ;naechster Track
track_wahl     = $A8        ;gewaehlter Playlisteneintrag - kann von interner Tracknummer abweichen

pause          = $A9        ;0 = Player spielt, 1 = Player in Pause
REU_Bank1      = $AA        ;Zwischenspeicher fuer REU-Bank, da sich diese mit der 1541u nicht direkt inkrementieren laesst
temp           = $AB
temp2          = $AC
temp3          = $AD
temp4          = $AE
ua_multi       = $AF        ;Multiplikator for Positionsbestimmung bei Ultimate Audio Tracks

tempmem        = $B0        ;Zwischenspeicher, der von verschiedenen Routinen genutzt werden kann (32 Bytes)
;ab $D0 frei


pos_im_track   = $F7        ;aktuelle Position im Track (2 Bytes: Pages+Baenke, gerundet), fuer Fortschrittsbalken
pos_infobar    = $F9        ;Position des Infoscrolltexts (2 Bytes)
divrest        = $FB        ;Rest der Division (3 Bytes)
pztemp         = $FE        ;Zwischenspeicher fuer Divisionsroutine

postrackanzeige = 1024+4*40+3     ;Startposition Trackanzeige im Bildschirmspeicher
posbalken       = 1024+6*40+20    ;Startposition Fortschrittsbalken im Bildschirmspeicher
posinfoleiste   = 1024+23*40+1    ;Startposition der Infoleiste im Bildschirmspeicher
posfehlermeldung= 1024+10*40+1    ;Startposition fuer Fehlermeldungen im Bildschirmspeicher
poskhzanzeige   = 1024+4*40+20    ;Startposition fuer Anzeige der Samplerate in kHz (abgerundet)
posbitanzeige   = 1024+4*40+27    ;Position fuer Anzeige der Bits pro Sample (4 oder 8)
posmonostereo   = 1024+4*40+33    ;Startposition für Anzeige Mono oder Stereo
posstatusanzeige= 1024+4*40+7     ;Startposition der Statusanzeigen
posplaypfeil    = 1024+10*40+3    ;Startposition des Playpfeils
postrackcursor  = 1024+10*40      ;Startposition des Trackcursors

;*** Charset und Screen
*=$3C00
!bin "charset.bin"
*=$4000
!bin "screen.bin"

;*** Startadresse BASIC-Zeile
*=$0801
        !byte $0c,$08,$e4,$07,$9e,$20,$32,$30,$36,$32,$00,$00,$00

;Systeminterrupts und Systemroms abschalten - diese erzeugen Tonstoerungen, wenn sie aktiv sind durch unerwuenschte Interrupts
 lda #0
 sta $DC0E
 sei
 lda #27
 sta $01
 ldx #0
 charcopy
 lda $D800,x
 sta $3800,x
 lda $D900,x
 sta $3900,x
 lda $DA00,x
 sta $3A00,x
 lda $DB00,x
 sta $3B00,x
 inx
 bne charcopy
 lda #$35
 sta $01
 
;Ultimate Audio ausschalten
 lda #0                    ;Ueberpruefung durch Spielen eines kurzen Samples: wenn kein Interrupt ausgeloest wird, ist UA nicht vorhanden
 sta UA+$00                ;Ultimate Audio stoppen (falls es noch aktiv ist)
 sta UA+$20
 
;16 (fuer 16 Bit) in Char 95 ($3AF8)
 lda #%00100110
 sta $3AF8
 lda #%01101001
 sta $3AF9
 lda #%10101000
 sta $3AFA
 lda #%00101110
 sta $3AFB
 lda #%00101001
 sta $3AFC
 lda #%00101001
 sta $3AFD
 lda #%00100110
 sta $3AFE
 lda #%00000000
 sta $3AFF
 
;Repeatsymbol in Char 126 ($3BF0)
 lda #%00100000 
 sta $3BF0
 lda #%00110000
 sta $3BF1
 lda #%01111010
 sta $3BF2
 lda #%10110001
 sta $3BF3
 lda #%10100001
 sta $3BF4
 lda #%10000001
 sta $3BF5
 lda #%01111110
 sta $3BF6
 lda #0
 sta $3BF7
 
;Aufnahmesymbol in Char 127 ($3BF8)
 lda #0
 sta $3BF8
 sta $3BFF
 lda #%00111100
 sta $3BF9
 sta $3BFE
 lda #%01111110
 sta $3BFA
 sta $3BFB
 sta $3BFC
 sta $3BFD
 
;Charset auf $3800 
lda #%00011110
sta $D018                

;Variableninitialisierung einmalig
lda #0
sta fpgasid_fehlt
sta disco_rahmen
sta disco_hinterg
sta statusanzeige
sta infobar
sta pos_infobar
sta pos_infobar+1
sta trackrepeat
sta tvnorm_falsch
sta starttrack
sta ua_reparse
lda #1
sta sidmanuell
sta statusanzeige
lda #$10
sta infospeed 

;Farben initialisieren
lda #6
sta hintergfarbe
sta $D021
lda #14
sta $D020 
sta textfarbe
ldx #0
loop_textfarbe_init
sta $D800,x
sta $D900,x
sta $DA00,x
sta $DB00,x
inx
bne loop_textfarbe_init


player_anzeigen            ;Einsprungpunkt fuer Playerreset (nach Fehler oder Aufnahmeende)

;FPGASID auf Configmode stellen (wirkungslos, wenn kein FPGASID vorhanden) und detektieren
 lda #$81                 ;Magic Bytes fuer Config mode
 sta $D419
 lda #$65
 sta $D41A
 lda #0                   ;Variable fpgasid_vorh initialisieren (bleibt 0, wenn kein FPGASID gefunden wird) 
 sta fpgasid_vorh
 lda #%10000000           ;"Identify" F51D aktivieren (wirkungslos, wenn kein FPGASID vorhanden)
 sta $D41E
 lda $D41A                ;pruefe $D41A und $D419 auf "F51D" - wird dieser Wert gelesen, ist ein FPGASID im C64
 cmp #$F5
 bne kein_fpgasid2
 lda $D419
 cmp #$1D
 bne kein_fpgasid2
 lda #1                   
 sta fpgasid_vorh 
kein_fpgasid2 

;Ultimate Audio detektieren (Ergebnis 0, wenn UA definitiv nicht vorhanden (Versionsregister $0x oder $FF), 1 wenn UA vorhanden (Version $10), 2 wenn unbekannt (Version nicht $10, aber auch nicht $0x oder $FF)
lda #0
sta ua_vorh
lda $DF21                 ;wenn im Versionsregister $0x oder $FF steht, wird davon ausgegangen, dass UA nicht aktiv ist
and #%11110000
beq ua_pruefung_ende
lda $DF21
cmp #$FF
beq ua_pruefung_ende
ldx #10
ua_pruefung_loop
lda #0                    ;Ueberpruefung durch Spielen eines kurzen Samples: wenn kein Interrupt ausgeloest wird, ist UA nicht vorhanden
sta UA+$00                ;Ultimate Audio stoppen (falls es noch aktiv ist)
sta UA+$20
lda #$FF
sta UA+$1F                ;Interrupt clear
lda $DF20                 ;wenn Interrupt Status Register nicht 0: Ultimate Audio nicht vorhanden
bne ua_pruefung_ende
lda #0
sta UA+$01                ;Lautstaerke auf 0, um Knackser zu vermeiden
sta UA+$09                ;Samplelaenge 2 (sehr kurz)
sta UA+$0A
sta $DF20                 ;Interrupt Status Register auf 0 (vermindert Risiko eines "False Positive"
lda #2
sta UA+$0B
lda #0
sta UA+$0E 
lda #130
sta UA+$0F
lda #%00000101            ;Ultimate Audio starten
sta UA+$00  
lda $D012                 ;warten, bis zwei Bildschirmzeilenwechsel statt gefunden haben
sta temp
ua_pruefung_loop_1
lda $D012
cmp temp
beq ua_pruefung_loop_1
sta temp
ua_pruefung_loop_2
lda $D012
cmp temp
beq ua_pruefung_loop_2
lda $DF20
cmp #1                     ;Interrupt Status Register sollte nun 1 sein, wenn nicht: UA ist nicht aktiv
bne ua_pruefung_ende
dex
bne ua_pruefung_loop
lda #1                     ;wenn alle Pruefungen erfolgreich waren ist UA aktiv 
sta ua_vorh
ua_pruefung_ende

;Variableninitialisierung
lda #0
sta pause
sta wechsel
sta infozeit
sta fpgasid_datei
sta cursor_pos
sta steuerung
sta rec_ueberlauf
sta rec_aufnahme
sta akt_takte
sta eingabemodus
sta ua_hq
lda #$FF
sta sidtyp_a
lda #50
sta te_blinkzeit

;Alternativplayliste initialisieren (zu Beginn in normaler Trackreihenfolge)
ldx #0
loop_alternativplaylist_init
txa
sta altern_playlist,x
inx
cmp #14
bne loop_alternativplaylist_init

;Screen kopieren
ldx #0
loop_screencopy
lda $4000,x
sta $0400,x
lda $4100,x
sta $0500,x
lda $4200,x
sta $0600,x
lda $4300,x
sta $0700,x
inx
bne loop_screencopy

;testen, ob PAL oder NTSC (Code von retroprogramming.de) Ergebnis $07 fuer NTSC, $06 fuer NTSC alt, $38 fuer PAL und Drean
 tvnorm_test
 lda $d011                          ;Warten, bis in $d011
 and #%10000000                     ;das höchste Bit gesetzt ist
 beq tvnorm_test                    ;...
 
 ldx #$00                           ;Bei 0 beginnen
 loop_tvnorm_test
 cpx $d012                          ;vergleiche $d012 mit X
 bcs loop_tvnorm_test               ;falls kleiner, warten
 ldx $d012                          ;aktuelle Zeile im X-Reg. merken
 lda $d011                          ;solange in $d011
 and #%10000000                     ;das höchste Bit gesetzt ist
 bne loop_tvnorm_test               ;wiederholen
 cpx #$38
 beq tvnorm_test_PAL
 lda #1
 jmp tvnorm_test_NTSC
 tvnorm_test_PAL
 lda #0
 tvnorm_test_NTSC
 sta tvnorm_c64 
 
 beq c64istPAL      ;ueberspringt NTSCfix, wenn PAL
 lda #$EA           ;aendert 2x "bit #$03" (6 Takte) in 4x "NOP" (8 Takte) an 4 Stellen beim Synchronisieren des Playerstarts (Bad-Line-Handling)
 sta NTSCfix1
 sta NTSCfix1+1
 sta NTSCfix1+2
 sta NTSCfix1+3 
 sta NTSCfix2
 sta NTSCfix2+1
 sta NTSCfix2+2
 sta NTSCfix2+3 
 sta NTSCfix3
 sta NTSCfix3+1
 sta NTSCfix3+2
 sta NTSCfix3+3 
 sta NTSCfix4
 sta NTSCfix4+1
 sta NTSCfix4+2
 sta NTSCfix4+3 
 c64istPAL
 
;Timer fuer Scrollgeschwindigkeit der Infoleiste einrichten
 lda #$ff                           ;LO-BYTE für
 sta $dc04                          ;Timer-A vom CIA-1 setzen
 lda #$1f                           ;und auch das HI-BYTE
 sta $dc05                          ;setzen
 lda #%00000001                     ;Timer-A im CIA-1
 sta $dc0e                          ;starten
 
 lda infospeed                      ;LO-BYTE für
 sta $dc06                          ;Timer-B vom CIA-1 setzen
 lda #$00                           ;und auch das HI-BYTE
 sta $dc07                          ;setzen
 lda #%01000001                     ;Timer-B im CIA-1
 sta $dc0f                          ;starten
 

!source "Breadampv05_REUFS_Parser.asm"

;pruefen, ob Infoleiste angezeigt wird
lda #0
sta infobar
lda maxtrack
cmp #13
bcs keine_infoleiste
jsr infoleiste_ein
keine_infoleiste

;los gehts
jsr player_init

lda starttrack
sta track        ;Track 1 ist intern #0 und damit beginnt die Wiedergabe (wenn vorher eine Aufnahme statt gefunden hat, wird der aufgenommen Track gespielt)
ldy #253         ;REU-Command -> !!! Y darf ab diesen Punkt während der Wiedergabe nicht mehr genutzt werden (außer in der Sample-Playerroutine)
ldx #0
stx starttrack   ;starttrack wieder auf 0 setzen fuer naechsten Playerreset
jmp trackwechsel ;ersten Track abspielen



;---------- Frontend im laufenden Betrieb --------------

;Tracknummer in 7-Segment-Anzeige
tracknranzeige
  jsr tracknranzeige_sub
  
  ;Playpfeil
  jsr playpfeil

  
  ;Metadaten ins Display
    ;Samplerate
    ldx track
    lda track_srate_z,x 
    sta poskhzanzeige
    lda track_srate_e,x
    sta poskhzanzeige+1
    ;Bits
    clc
    lda track_bits,x
    cmp #16
    beq meta_16bit
    adc #$30
    sta posbitanzeige
    jmp meta_16bit_ende
    meta_16bit
    lda #95
    sta posbitanzeige
    meta_16bit_ende
    ;Mono/Stereo 
    lda track_stereo,x 
    bne Anzeige_Stereo 
    lda #77                ;Anzeige "Mono"
    sta posmonostereo
    lda #15
    sta posmonostereo+1
    sta posmonostereo+3
    lda #14
    sta posmonostereo+2 
    jmp Anzeige_Mono   
    Anzeige_Stereo 
    lda #178              ;Anzeige "Stereo"
    sta posmonostereo
    lda #179
    sta posmonostereo+1    
    lda #180
    sta posmonostereo+2    
    lda #181
    sta posmonostereo+3    
    Anzeige_Mono
    
  ;Format-/Statusanzeigen
    ;SID-Typ (in Metadaten der Datei oder manuell eingestellt)
    jsr statusanzeigen     ;ausgelagert, um es im Programm weitere Male nutzen zu koennen

;Trackendeueberwachung (teu)
trackendeueberwachung
ldx track
lda ua_vorh          ;pruefe, ob Ultimate Audio vorhanden und Track im UA-Format, wenn ja: pruefe, ob ein Trackendeinterrupt ausgeloest wurde - wenn ja: springe direkt zum naechsten Track
beq teu_ua_nicht_vorh
lda track_sidtyp,x
cmp #7
bne teu_ua_nicht_vorh
lda $DF20
bne teu_trackwechsel
teu_ua_nicht_vorh
lda REU_Bank1
sta temp2
lda REU_Hi
sta temp 
bne sub              ;wenn REU_Hi 0 ist, lade REU_Bank1 nochmal, denn es koennte zwischenzeitlich eine Erhoehung um 1 passiert sein
 lda REU_Bank1
 sta temp2
 sub
  sec
  lda trackende_m,x
  sbc temp
  sta temp3
  lda trackende_h,x
  sbc temp2 
 ;pruefe, ob Ergebnis der Subtraktion $0007 oder kleiner ist
  bne teu_ende       ;ungleich Null = Ende
  lda temp3
  cmp #$08           ;wird negativ, wenn Wert <= $07 ist
  bcs teu_ende       ;keine Ueberwachung, wenn noch genug Abstand zum Trackende ist
  lda pause
  bne teu_trackwechsel  ;verhindert Einfrieren bei Pause kurz vor Trackende -> geht dann direkt zum naechsten Track
 ueberwachung1   ;pruefen, bis weniger als 256 Byte bis Trackende
   lda REU_Hi
   sta temp2
   lda REU_Lo
   sta temp 
   bne sub1              ;wenn REU_Lo 0 ist, lade REU_Hi nochmal, denn es koennte zwischenzeitlich eine Erhoehung um 1 passiert sein
   lda REU_Hi
   sta temp2
   sub1
    sec
    lda trackende_l,x    ;Lo-Byte des Ergebnis wird nicht gespeichert, da es fuer die Pruefung nicht benoetigt wird
    sbc temp
    lda trackende_m,x 
    sbc temp2
    bne ueberwachung1
   lda track_sidtyp,x ;bei Ultimate Audio nicht aufs letze Byte warten, sondern auf das Setzen des Interruptregisters 
   cmp #7             ;(verhindert vorzeitigen Wiedergabeabbruch bei leichter Asynchronitaet C64 <-> Ultimate Audio)
   beq ueberwachung_ua
 ueberwachung2   ;warten, bis Mid- und Lo-Byte der aktuellen Playerposition gleich oder größer Mid/Hi vomTrackende
    lda REU_Hi
    cmp temp2
    bne ueberwachung2
    lda REU_Lo 
    cmp temp 
    bcc ueberwachung2
    jmp teu_trackwechsel
 ueberwachung_ua
    lda $DF20     ;warten bis das Interruptregister von Ultimate Audio das Wiedergabeende meldet (nur bei Ultimate Audio-Tracks)
    bne teu_trackwechsel
    beq ueberwachung_ua
 teu_trackwechsel
    lda trackrepeat
    bne teu_trackrepeat
    lda naechst_track
    jmp trackwechsel
    teu_trackrepeat
    lda track_wahl
    jmp trackwechsel
teu_ende

  lda wechsel            ;im Wechsel Spielzeit und Fortschrittsbalken berechnen (reduziert Rechenzeit pro Durchlauf)
  eor #1
  sta wechsel
  bne spielzeit
  jmp fortschrittsbalken

;Spielzeitanzeige
  spielzeit
  ldx track 
  lda REU_Bank1
  sta temp2
  lda REU_Hi
  sta temp 
  bne sub2              ;wenn REU_Hi 0 ist, lade REU_Bank1 nochmal, denn es koennte zwischenzeitlich eine Erhoehung um 1 passiert sein
  lda REU_Bank1
  sta temp2
  sub2  
  sec
 
  
  lda temp                 ;abgespielte Bytes berechnen und in Dividend laden
  sbc trackstart_m,x
  sta dividend+1
  sta pos_im_track         ;Ergebnis auch in pos_im_track sichern, wird fuer den Fortschrittsbalken noch gebraucht
  lda temp2 
  sbc trackstart_h,x
  sta dividend+2
  sta pos_im_track+1
  lda #0
  sta dividend 
  jsr spielzeit_sub1        ;weiter gehts im Unterprogramm (wurde ausgelagert, um die Anzeige auch im FPGASID-Recorder nutzen zu koennen)

   ;Timer fuer Statusanzeige, Discomodus und Trackcursor
   lda status_sek
   cmp divrest
   beq status_zeit_nicht_senken   ;immer noch die gleiche Sekunde
    stx status_sek       
     lda disco_rahmen             ;den "Statustimer" auch fuer den Discomodus nutzen
     beq disco_hintergrund_check
     inc $D020
     disco_hintergrund_check
     lda disco_hinterg
     beq disco_auswertung_ende
     disco_hintergrundfarbe_plus
     lda hintergfarbe
     clc
     adc #1
     and #$0F 
     sta hintergfarbe
     cmp textfarbe                ;verhindern, dass Textfarbe = Hintergrundfarbe den Player verschwinden laesst
     beq disco_hintergrundfarbe_plus
     sta $D021 
     disco_auswertung_ende
    lda cursor_zeit
    beq cursor_auswertung_ende
    sec
    sbc #1
    sta cursor_zeit
    bne cursor_auswertung_ende
    jsr trackcursor_aus
    cursor_auswertung_ende
   lda statusanzeige
   cmp #2
   beq status_zeit_nicht_senken   ;Statusanzeige dauerhaft eingeschaltet
   lda status_zeit
   beq status_zeit_nicht_senken   ;Restzeit schon 0 und Statusanzeige abgeschaltet
   sec
   sbc #1
   sta status_zeit
   bne statusanzeige_an_lassen
   jsr statusanzeige_aus
   status_zeit_nicht_senken
   statusanzeige_an_lassen
  jsr spielzeit_sub2        ;weiter gehts im Unterprogramm (wurde ausgelagert, um die Anzeige auch im FPGASID-Recorder nutzen zu koennen)   
   
jmp tastaturabfrage

;Fortschrittsbalken
  fortschrittsbalken
  lda #0
  sta dividend      ;Low-Byte des Dividend und High-Byte des Divisors hier immer 0
  sta divisor+2
  
  lda pos_im_track+1
  sta dividend+2 
  lda pos_im_track
  sta dividend+1
  
  ldx track
  lda tracklaenge_h,x 
  sta divisor+1
  lda tracklaenge_m,x
  sta divisor
  jsr div24
  
  lda dividend+1
  beq wert_ok
  lda #$ff         ;bei Ueberlauf A auf ff setzen
  jmp wert_fehler
  wert_ok
  lda dividend     ;wenn kein Ueberlauf A auf dividend setzen
  wert_fehler
  sta temp 
  lsr
  lsr
  lsr
  lsr
  sta temp2        ;Spalte, in der der Balken beginnt (0-15)
  lda temp
  and #%00001110
  lsr
  sta temp3        ;Pixelverschub des Balkens (0-7)
  
   ;Punkte vor dem Balken
   lda temp2
   beq balken_spalte_0
   tax
   dex
   balken_spalte
    lda #216       ;Zeichen Punkte oben
    sta posbalken,x
    lda #233       ;Zeichen Punkte unten
    sta posbalken+40,x
    dex
    cpx #$ff       
    bne balken_spalte
   balken_spalte_0
   
   ;Balken
   ldx temp2
   lda temp3
   clc
   adc #200        ;Zeichen fuer Pixelverschub 0, oben links
   sta posbalken,x
   lda temp3
   adc #217        ;Zeichen fuer Pixelverschub 0, unten links
   sta posbalken+40,x
   lda temp3
   adc #208        ;Zeichen fuer Pixelverschub 0, oben rechts
   sta posbalken+1,x
   lda temp3
   adc #225        ;Zeichen fuer Pixelverschub 0, unten rechts
   sta posbalken+41,x
   
   ;Punkte hinter dem balken
   ldx temp2
   cpx #15
   beq balken_spalte_15
   balken_spalte_y
    lda #216       ;Zeichen Punkte oben
    sta posbalken+2,x
    lda #233       ;Zeichen Punkte unten
    sta posbalken+42,x 
    inx
    cpx #15
    bne balken_spalte_y
   balken_spalte_15
   
;Infoleiste
 lda infobar
 beq tastaturabfrage      ;ueberspringen, wenn Infoleiste aus
 lda #%00000010           ;pruefe, ob Timerunterlauf passiert ist, wenn ja: scrolle ein Zeichen weiter
 bit $dc0d
 beq tastaturabfrage
 
 inc pos_infobar
 bne pruefe_pos_infobar
 inc pos_infobar+1
 pruefe_pos_infobar
  sec                     ;vergleiche mit Wert $8F5 (Dez 2293 = 2331-38 -> 2331=Laenge des Infotext), Berechnung nur fuer Carry-Flag
  lda #$F5
  sbc pos_infobar
  lda #$8
  sbc pos_infobar+1
  bcs zeige_infoleiste
  lda #0                  ;wenn Positionszeiger zu gross, setze ihn auf 0 zurueck
  sta pos_infobar
  sta pos_infobar+1
 zeige_infoleiste
  clc                     ;Startadresse im Infotext berechnen und in adr1_l/h speichern
  lda #<infotext
  adc pos_infobar
  sta adr_infobar+1 
  lda #>infotext
  adc pos_infobar+1
  sta adr_infobar+2
  ldx #0                 ;Infotext laden und auf Bildschirm ausgeben
  ausgabe_infotext
   adr_infobar lda $0000,x ;Platzhalter fuer Infobaradresse
   sta posinfoleiste,x
   inx
   cpx #38
   bne ausgabe_infotext
  
   
!source "Breadampv05_Tastaturfunktionen.asm"


;Trackwechsel (Tracknummer im Akku)
trackwechsel
cmp maxtrack
bpl ungueltige_tracknummer_1 ;Tracknummer gewaehlt, die groesser als Anzahl der Tracks ist
jmp gueltige_tracknummer
ungueltige_tracknummer_1
jmp ungueltige_tracknummer
gueltige_tracknummer
sta track_wahl
tax
lda altern_playlist,x
sta track
tax
lda #%00000001           ;IRQ fuer Timer-A 
sta $dd0d                ;deaktivieren, um Fehler beim Trackwechsel zu vermeiden
lda #0
sta UA+$00                ;Ultimate Audio stoppen (wirkungslos, wenn kein UA aktiv)
sta UA+$20
lda ua_track_mult,x
sta ua_multi
lda track_takte,x
cmp akt_takte
beq kein_sampleratewechsel
jsr player_samplerate_wechseln
kein_sampleratewechsel 
ldx track
lda trackstart_h,x       ;neue REU-Position fuer den Player
sta REU_Bank1
sta REU_Bank
lda trackstart_m,x
sta REU_Hi
lda trackstart_l,x
sta REU_Lo
lda track_sidtyp,x       ;pruefe, ob beim Trackwechsel der Player neu initialisiert werden muss, wenn nicht -> abkuerzen
cmp #7                   ;bei Ultimate Audio immer reinitialisieren, da hier die API gesteuert wird
beq trackwechsel_reinitialisierung
cmp sidtyp_a
bne trackwechsel_reinitialisierung
lda track_init_h,x
cmp track_init_a_h
bne trackwechsel_reinitialisierung
lda track_init_l,x
cmp track_init_a_l
beq trackwechsel_keine_reinitialisierung
trackwechsel_reinitialisierung
lda track_sidtyp,x
sta sidtyp_a
lda track_init_l,x 
sta initsprung+1
sta track_init_a_l
lda track_init_h,x
sta initsprung+2
sta track_init_a_h
initsprung jsr $0000               ;$0000 ist nur Platzhalter, die Zieladresse wird ueber die Variable track_init_l/h festgelegt
lda REU_Hi
trackwechsel_keine_reinitialisierung
lda track_player_l,x               ;Adresse des NMI
sta $FFFA
lda track_player_h,x
sta $FFFB
lda #%10000001           ;IRQ fuer Timer-A 
sta $dd0d                ;aktivieren

lda #0                   ;Pauseflag resetten
sta pause
sta status_sek           ;beim Trackbeginn muss status_sek 0 sein, sonst bleibt die Statusanzeige bis zu einer Minute an auch wenn sie nur kurz an sein soll

lda track_takte,x        ;aktuelle Takte zw. den Samples speichern (fuer Vergleich beim Trackwechsel)
sta akt_takte

clc
lda track_wahl
adc #1                   ;+1, da maxtrack 1 hoeher ist als der hoechste logische Track
cmp maxtrack
bne track_plus_1
lda #0
sta naechst_track
jmp tracknranzeige
track_plus_1
sta naechst_track
jmp tracknranzeige
ungueltige_tracknummer
jmp trackendeueberwachung


      
;Sampleplayer einrichten und starten (gemaess Dokument von Mahoney)
player_init
;SID initialisieren
lda #$0f ;Setup attack=0 and decay=15
sta $d405
sta $d40c
sta $d413
lda #$ff ;Setup all sustain&release to 15
sta $d406
sta $d40d
sta $d414
lda #$49 ;Waveform is square, test bit set
sta $d404
sta $d40b
sta $d412
lda #$ff ;Filter cutoff as high as possible
sta $d415
sta $d416
lda #$03 ;Enable voice 1 and 2 through filter
sta $d417

lda #$0f ;Setup attack=0 and decay=15
sta $d505
sta $d50c
sta $d513
lda #$ff ;Setup all sustain&release to 15
sta $d506
sta $d50d
sta $d514
lda #$49 ;Waveform is square, test bit set
sta $d504
sta $d50b
sta $d512
lda #$ff ;Filter cutoff as high as possible
sta $d515
sta $d516
lda #$03 ;Enable voice 1 and 2 through filter
sta $d517

;REU, RAM und Register initialisieren
lda #%01111111                     ;alle IRQs
sta $dd0d                          ;deaktivieren
    lda #$00
    sta REU_Lo
    sta REU_Hi
    sta REU_Bank
    sta REU_BytesHi
    sta REU_Bank1
    lda #$01
    sta REU_BytesLo
    rts
    
init_stereo_8bit
    lda #$D4
    sta REU_C64Hi
    lda #$18
    sta REU_C64Lo

    lda fpgasid_vorh
    beq init_stereo_8bit_kein_fpgasid
    ;lda #%10000000               ;FPGASID: identify F51D, Dual Mono (aus beiden Lautsprechern)
    ;sta $D41E
    lda #0                       ;FPGASID Register Reset (nötig, damit er nicht verstummt beim Schalten von Stereo auf Mono)
    sta $D404                    
    sta $D40B
    sta $D412
    lda #73
    sta $D404
    sta $D40B
    sta $D412
    lda #%10000010               ;FPGASID: identify F51D, Stereo (2ter SID auf $D500)
    sta $D41E
    lda track_sidtyp,x           ;automatische Einstellung auf korrekten SID (sofern Metadaten verfuegbar, sonst manuell)
    cmp #1
    beq init_stereo_8bit_6581
    cmp #2
    beq init_stereo_8bit_8580
    cmp #5
    beq init_stereo_8bit_6581_8580
    cmp #6
    beq init_stereo_8bit_8580_6581
    jmp Fehler_in_der_Datei    ;falls ein ungueltiges Stereoformat in den Metadaten steht

    init_stereo_8bit_6581
    lda #%01000000             ;Digifix deaktivieren + Einstellung auf 6581
    sta $D41F
    sta $D51F
    rts
    init_stereo_8bit_8580
    lda #%01111111             ;Digifix deaktivieren + Einstellung auf 8580
    sta $D41F
    sta $D51F
    rts
    init_stereo_8bit_6581_8580
    lda #%01000000             ;Digifix deaktivieren + Einstellung auf 6581 (SID1)
    sta $D41F
    lda #%01111111             ;Digifix deaktivieren + Einstellung auf 8580 (SID2)
    sta $D51F
    rts
    init_stereo_8bit_8580_6581
    lda #%01111111             ;Digifix deaktivieren + Einstellung auf 8580 (SID1)
    sta $D41F
    lda #%01000000             ;Digifix deaktivieren + Einstellung auf 6581 (SID1)
    sta $D51F
    init_stereo_8bit_kein_fpgasid
    rts
      
init_mono_8bit
    lda track_sidtyp,x
    cmp #4 
    bne init_mono_8bit_kein_digimax
    lda digimax_adr_h          ;DigiMax-Adresse
    sta REU_C64Hi
    lda digimax_adr_l
    sta REU_C64Lo  
    lda #$FF    ;CIA 2 Port B alle Bits auf Ausgang
    sta $DD03  
    jmp init_mono_8bit_kein_fpgasid
    init_mono_8bit_kein_digimax
    lda #$D4
    sta REU_C64Hi
    lda #$18
    sta REU_C64Lo
    lda fpgasid_vorh
    beq init_mono_8bit_kein_fpgasid
    lda #%10000000              ;FPGASID: identify F51D, Dual Mono (aus beiden Lautsprechern)
    sta $D41E 
    lda #0                      ;FPGASID Register Reset (nötig, damit er nicht verstummt beim Schalten von Stereo auf Mono)
    sta $D404                    
    sta $D40B
    sta $D412
    lda #73
    sta $D404
    sta $D40B
    sta $D412
    lda track_sidtyp,x          ;automatische Einstellung auf korrekten SID (sofern Metadaten verfuegbar, sonst manuell)
    bne init_mono_8bit_autosid
    lda sidmanuell
    beq init_mono_8bit_6581
    jmp init_mono_8bit_8580
    init_mono_8bit_autosid
    lda track_sidtyp,x
    cmp #1
    bne init_mono_8bit_8580
    init_mono_8bit_6581
    lda #%01000000             ;Digifix deaktivieren + Einstellung auf 6581
    sta $D41F
    rts
    init_mono_8bit_8580
    lda #%01111111             ;Digifix deaktivieren + Einstellung auf 8580
    sta $D41F
    init_mono_8bit_kein_fpgasid
    rts
    
init_mono_4bit
    lda #1
    sta halbbyte                  ;mit 1 starten, damit erster Toggle es auf 0 (=linkes Halbbyte) bringt             
    lda #>sample
    sta REU_C64Hi
    lda #<sample
    sta REU_C64Lo
    lda fpgasid_vorh
    beq init_mono_4bit_kein_fpgasid
     lda #%10000000               ;FPGASID: identify F51D, Dual Mono (aus beiden Lautsprechern)
     sta $D41E
     lda #%01000000               ;Digifix deaktivieren + Einstellung auf 6581
     sta $D41F
     lda #0                       ;FPGASID Register Reset (nötig, damit er nicht verstummt beim Schalten von Stereo auf Mono)
     sta $D404                    
     sta $D40B
     sta $D412
     lda #$49
     sta $D404
     sta $D40B
     sta $D412
    init_mono_4bit_kein_fpgasid
    rts

init_fpgasid_mono
    lda #$0F                     ;Lautstaerke aufdrehen, sonst hat man bei dieser Abspielmethode Stille
    sta $D418
    sta $D518                    ;der FPGASID will auch den zweiten SID aufgedreht haben, sonst ist der rechte Lautsprecher ohne Ton
    
    lda #%10000000               ;FPGASID: identify F51D, Dual Mono (aus beiden Lautsprechern)
    sta $D41E
    lda #$FF                     ;Digifix aktivieren + Einstellung auf 8580
    sta $D41F

    lda #$D4                    ;Adresse des Digifix-Registers des FPGASID als REU-Transfer-Ziel
    sta REU_C64Hi
    lda #$1D
    sta REU_C64Lo  
    rts 
    
init_fpgasid_stereo
    lda #$0F                     ;Lautstaerke aufdrehen, sonst hat man bei dieser Abspielmethode Stille
    sta $D418
    sta $D518 

    lda #%10000010               ;FPGASID: identify F51D, Stereo (2ter SID auf $D500)
    sta $D41E
    
    lda #$FF                      ;Digifix aktivieren (+Einstellung auf 8580, die ist hier aber egal)
    sta $D41F
    sta $D51F
    
    lda #$D4                     ;Adresse des Digifix-Registers des FPGASID als REU-Transfer-Ziel
    sta REU_C64Hi
    lda #$1D
    sta REU_C64Lo 
    rts 
    
init_ultimate_audio_mono
    lda #0
    sta UA+$00  ;Ultimate Audio stoppen
    lda #$3F    ;Lautstärke
    sta UA+$01
    lda #$07    ;Balance ($07 = Mitte)
    sta UA+$02
    lda #$01    ;Startadresse $01000000 + REU-Start
    sta UA+$04
    lda trackstart_h,x   
    sta UA+$05
    lda trackstart_m,x
    sta UA+$06
    lda trackstart_l,x
    sta UA+$07
    lda tracklaenge_h,x    ;Samplelänge
    sta UA+$09
    lda tracklaenge_m,x
    sta UA+$0A
    lda tracklaenge_l,x
    sta UA+$0B
    lda ua_track_T_h,x    ;Samplerate 6.250.000 / Samplerate = Wert (aufgerundet) 
    sta UA+$0E
    lda ua_track_T_l,x
    sta UA+$0F
    lda #$FF    ;Interrupt Clear
    sta UA+$1F

    lda track_bits,x
    cmp #16
    beq init_ultimate_audio_mono_16bit
    lda #%00000101   ;Ultimate Audio starten
    sta UA+$00   
    rts
    init_ultimate_audio_mono_16bit
    lda #%00010101   ;Ultimate Audio starten im 16-Bit-Modus
    sta UA+$00   
    rts
    
init_ultimate_audio_stereo  ;Offset $00 = linker Kanal, $20 = rechter Kanal
    lda track_bits,x
    cmp #16
    beq init_ua_st_16
    lda #1
    sta ua_versatz_st
    jmp init_ua_st_16_ende
    init_ua_st_16
    lda #2
    sta ua_versatz_st
    init_ua_st_16_ende
    lda #0
    sta UA+$00  ;Ultimate Audio stoppen
    sta UA+$20
    lda #$3F    ;Lautstärke
    sta UA+$01
    sta UA+$21
    lda #$00    ;Balance ($00 = Links, $0F = right)
    sta UA+$02
    lda #$0F
    sta UA+$22
    lda #$01    ;Startadresse $01000000 + REU-Start
    sta UA+$04
    sta UA+$24
    lda trackstart_h,x   
    sta UA+$05
    sta UA+$25
    lda trackstart_m,x
    sta UA+$06
    sta UA+$26
    lda trackstart_l,x
    sta UA+$07
    clc
    adc ua_versatz_st
    sta UA+$27
    lda tracklaenge_h,x    ;Samplelänge
    sta UA+$09
    sta UA+$29
    lda tracklaenge_m,x
    sta UA+$0A
    sta UA+$2A
    lda tracklaenge_l,x
    sta UA+$0B
    sta UA+$2B
    lda ua_track_T_h,x    ;Samplerate 6.250.000 / Samplerate = Wert (aufgerundet) 
    sta UA+$0E
    sta UA+$2E
    lda ua_track_T_l,x
    sta UA+$0F
    sta UA+$2F
    lda #$FF    ;Interrupt Clear
    sta UA+$1F
    sta UA+$3F

    lda track_bits,x
    cmp #16
    beq init_ultimate_audio_stereo_16bit
    lda #%01000101   ;Ultimate Audio starten
    sta UA+$00    
    sta UA+$20
    rts
    init_ultimate_audio_stereo_16bit
    lda #%01010101   ;Ultimate Audio starten im 16-Bit-Modus
    sta UA+$00    
    sta UA+$20
    rts


!align 255,0                       ;verhindern, dass die Player ueber die Pagegrenze gehen und dadurch ein zusaetzlicher Takt benoetigt wird
player_samplerate_wechseln 
lda #0
sta $dd0E
zeile = 251                      ;in dieser Zeile beginnt die Synchronisierung (der Player soll soweit moeglich "zwischen den Badlines des VIC" spielen
warte
lda $D012
cmp #zeile
bne warte
ldx #8
loop_z0
dex
bne loop_z0
nop
nop
NTSCfix1 bit $03
bit $03
lda $D012
cmp #zeile
beq sync_kurz1
nop
jmp sync_teil2

sync_kurz1
nop
bit $03
jmp sync_teil2

sync_teil2
ldx #7
loop_z1
dex
bne loop_z1
NTSCfix2 bit $03
bit $03
bit $03
bit $03
lda $D012
cmp #zeile+1
beq sync_kurz2
nop
jmp sync_teil3
sync_kurz2
bit $03
jmp sync_teil3

sync_teil3
ldx #8
loop_z2
dex
bne loop_z2
NTSCfix3 bit $03
bit $03
nop
lda $D012
cmp #zeile+2
beq sync_teil4

sync_teil4
ldx #9
loop_z3
dex
bne loop_z3
nop
NTSCfix4 bit $03
bit $03
lda $D012
cmp #zeile+3
beq sync_ende
sync_ende                ;Synchronisierung Ende 

nop                      ;die nops dienen der "Feinabstimmung" des Initialisierungsstarts
nop
;nop
ldx track

lda track_takte,x                  ;LO-BYTE fuer
sta $dd04                          ;Timer-A vom CIA-2 setzen
lda #$00                           ;und auch das HI-BYTE
sta $dd05                          ;setzen

 
lda #%00000001                     ;Timer-A im CIA-2
sta $dd0e                          ;starten
rts

!align 255,0                       ;verhindern, dass die Player ueber die Pagegrenze gehen und dadurch ein zusaetzlicher Takt benoetigt wird
player_stereo     ;Player Stereo, 8 Bit

         sty REU_CMD               ;REU-Fetch (Y immer 253 - ausschließlich fuer REU-Fetch reserviert !)
         inc REU_Lo
         inc REU_C64Hi             ;auf zweiten SID wechseln
         sty REU_CMD
         dec REU_C64Hi             ;zurueck zum ersten SID
         inc REU_Lo
         bne exit
         inc REU_Hi
         bne exit
         inc REU_Bank1
         ldy REU_Bank1
         sty REU_Bank 
         ldy #253                  ;geliehenes Y-Register wieder auf 253 setzen !
         exit
 bit $dd0d                         ;IRQs bestaetigen
         
rti                                ;und zurueck


player_mono_8bit    ;Player Mono, 8 Bit
         sty REU_CMD               ;REU-Fetch (Y immer 253 - ausschließlich fuer REU-Fetch reserviert !)
         inc REU_Lo
         bne exit2
         inc REU_Hi
         bne exit2
         inc REU_Bank1
         ldy REU_Bank1
         sty REU_Bank 
         ldy #253                  ;geliehenes Y-Register wieder auf 253 setzen !
         exit2 
 bit $dd0d                         ;IRQs bestaetigen
rti                                ;und zurueck

player_mono_4bit    ;Player Mono, 4 Bit
pha                                ;Akku
 lda $dd0d                         ;IRQs bestaetigen
  lda halbbyte
  eor #1                           ;toggelt zwischen linken (=0) und rechten (=1) Halbbyte
  sta halbbyte
    bne vierbit_halbbyte_rechts
         sty REU_CMD               ;REU-Fetch (Y immer 253 - ausschließlich fuer REU-Fetch reserviert !)
         lda sample                ;samplewert 4 Bit nach rechts, um das linke Halbbyte als Wert $0-$F zu erhalten
         lsr
         lsr
         lsr
         lsr
         sta $D418                 ;Wert ins Lautstaerkeregister des SID 
         jmp exit3
     vierbit_halbbyte_rechts
         lda sample 
         nop                       ;Verzoegerung zwecks Sampletiming
         nop
         nop
         nop
         nop
         and #%00001111            ;linkes Halbbyte ausmaskieren, um das rechte Halbbyte als Wert $0-$F zu erhalten
         sta $D418                 ;Wert ins Lautstaerkeregister des SID 
         inc REU_Lo
         bne exit3
         inc REU_Hi
         bne exit3
         inc REU_Bank1
         lda REU_Bank1
         sta REU_Bank
exit3 
pla                                ;Register vom Stack holen
rti                                ;und zurueck

player_ultimate_audio  ;"Player" fuer Ultimate Audio (zaehlt nur die Adresse hoch als "Positionsbestimmung")
    pha
         clc
         lda REU_Lo 
         adc ua_multi
         sta REU_Lo
         lda REU_Hi
         adc #0
         sta REU_Hi
         lda REU_Bank1
         adc #0
         sta REU_Bank1
         sta REU_Bank
    pla    
 bit $dd0d                         ;IRQs bestaetigen
rti                                ;und zurueck
 
!source "Breadampv05_Unterroutinen_und_Daten.asm"

