;Breadampv05_REUFS_Parser.asm
;----------------------------
;
;This file is required to be able to compile the main file "BREADAMPv05.asm". It cannot be compiled on its own.
;
;Diese Datei ist nötig, um die Hauptdatei "BREADAMPv05.asm" kompilieren zu können. Sie kann nicht eigenständig kompiliert werden.


;*** REU-FS auslesen
lda #0
sta REU_Bank
sta REU_Hi
sta REU_Lo
sta REU_BytesLo
sta REU_C64Lo
lda #3
sta REU_BytesHi
lda #>header
sta REU_C64Hi
lda #253
sta REU_CMD  ;Es werden $300 Bytes des REU-Anfangs nach $C000 im C64 verschoben (Metadaten+ein bisschen ueberhang)

;pruefen, ob die ersten Bytes des REU-Files den Text "BREADAMP" enthalten
ldx #0
pruefe_header_breadamp 
      lda header,x
      cmp text_breadamp,x
      beq pruefe_header_breadamp_ok
      jmp Fehler_kein_breadamp_file
      pruefe_header_breadamp_ok
      inx
      cpx #08
      bne pruefe_header_breadamp 

;Version der BREADAMP-Datei pruefen (Fehler wenn ungleich 02/03 = $30 $32/$33 ASCII/Screencodes)
      lda header+8
      sta text_Fehler_falsche_version_2     ;Versionsnummer in Fehlermeldung injizieren
      sta text_Fehler_falsche_version_6
      ldx header+9
      stx text_Fehler_falsche_version_2+1
      stx text_Fehler_falsche_version_6+1
      cmp #$30
      bne pruefe_version_fehler
      cpx #$32
      beq pruefe_version_kein_fehler
      cpx #$33
      beq pruefe_version_kein_fehler
      cpx #$35
      beq pruefe_version_kein_fehler
      pruefe_version_fehler
      jmp Fehler_falsche_Version
      pruefe_version_kein_fehler
      
      
      

;*** REU-FS parsen

  ;Staradresse im Bildschirmspeicher fuer Tracknummern in Playlist
  lda #$05      
  sta adr1_h
  lda #$6C
  sta adr1_l

  ;Startadresse der Trackinfos
  metadaten_neu_parsen
  lda header+$0F       ;Anzahl REUFS-Eintraege
  asl                  ;4x asl = Multiplikation mit 16 = 1 Zeile im Hex-Editor HxD
  asl
  asl
  asl
  adc #$0B             ;Adresse High, Bank wird nicht ausgelesen, da immer 0
  tax
  lda header,x
  sta trackinfos_h
  adc #>header
  sta adr2_h
  inx                  ;Adresse Low
  lda header,x
  sta adr2_l
  sta trackinfos_l
  lda ua_reparse       ;beim Neuparsen aufgrund Formatumschaltung nicht neu zu parsenden Teil ueberspringen 
  beq kein_neu_parsen
  ldx #0
  jmp metadaten_neu_parsen_1
  kein_neu_parsen

lda #0
sta track
lda #1
sta naechst_track
loop_parse_REU_FS
asl
asl
asl
asl
tay        ;in Y ist jetzt die Startadresse fuer den zu parsenden REU-FS eintrag z.B. fuer Eintrag 2 die $20 (+headerstart)
  
jsr tvnorm_init_c64

  ;pruefen, ob REU-FS-Eintrag mit "TRACK " beginnt
  ldx #0
  pruefe_header_track   
   lda header,y
   cmp text_track,x
   beq pruefe_header_track_ok
     ;wenn Eintrag nicht mit "TRACK " beginnt, prüfe, ob Eintrag "reserved entry  " ist, wenn ja, Ende des Auslesens, wenn nein, Fehler
     lda track
     bne pruefe_weiter                     ;wenn schon der erste Eintrag nicht mit TRACK beginnt, dann springe in den FPGASID-Recorder
     lda tvnorm_c64                        ;eine leere Datei erhält in den Metadaten die erkannte TV-Norm des C64
     sta track_tvnorm,x 
     lda #0
     sta maxtrack
     lda fpgasid_vorh
     bne REU_parser_fpgasid_vorh
     jmp Fehler_leere_Breadamp_Datei_kein_FPGASID
     REU_parser_fpgasid_vorh
     jmp fpgasid_recorder
     pruefe_weiter
     ldx #0
     pruefe_header_reserved_entry
      lda header,y
      cmp text_reserved_entry,x
      beq pruefe_header_reserved_entry_ok
       jmp Fehler_in_der_Datei
      pruefe_header_reserved_entry_ok
      inx
      iny
      cpx #16
     bne pruefe_header_reserved_entry
     jmp reserved_entry_gefunden 
   pruefe_header_track_ok
   inx
   iny
   cpx #6
  bne pruefe_header_track
  
   ;neue Bildschirmspeicheradresse berechnen    
   lda adr1_l
   clc
   adc #37  ;+1 Zeile (nur 37 statt 40, da der Wert spaeter nochmal um 1 und 2 erhoeht wird)
   sta adr1_l
   bcc kein_Uebertrag
   inc adr1_h
   kein_Uebertrag
  
   ;Tracknummer in Bildschirmspeicher und in Variablen tracknr_z und tracknr_e
   lda header,y  ;Pos 6 und 7 ($0+$6 und $1+$6) ist die Tracknummer gespeichert
   ldx #0
   sta (adr1_l,x)
   sec
   sbc #$30        ;$30 ist die Null, $31 die 1 usw.
   ldx track
   sta tracknr_z,x
   ldx #0
   inc adr1_l      ;eine Spalte weiter, Pruefung auf uebertrag nicht noetig, da er hier nie passiert
   lda header+1,y
   sta (adr1_l,x)
   sbc #$30
   ldx track
   sta tracknr_e,x

   
   ;Parsen der Trackadressen

   ldx track
   lda header+4,y   ;Pos $A bis $C ($4+$6 bis $6+$6) ist die Startadresse des Tracks gespeichert
   sta trackstart_h,x  ;die 3 Bytes fuer Startadresse des Tracks ins entsprechende "Array" kopieren
   lda header+5,y
   sta trackstart_m,x
   lda header+6,y
   sta trackstart_l,x
   
   ldx track
   lda header+7,y    ;Pos $D bis $F ($7+$6 bis $9+$6) ist die Laenge des Tracks gespeichert
   sta tracklaenge_h,x  ;die 3 Bytes fuer Laenge des Tracks ins entsprechende "Array" kopieren
   lda header+8,y
   sta tracklaenge_m,x
   lda header+9,y
   sta tracklaenge_l,x
   
   ;Berechnen der Endadresse des Tracks
   ldx track
   clc
   lda trackstart_l,x      ;mit dem kleinstwertigsten Byte beginnen
   adc tracklaenge_l,x
   sta trackende_l,x
   lda trackstart_m,x
   adc tracklaenge_m,x
   sta trackende_m,x
   lda trackstart_h,x
   adc tracklaenge_h,x
   sta trackende_h,x

   
   ;Trackname in Bildschirmspeicher kopieren und Metadaten parsen
   ldy #0
   inc adr1_l
   inc adr1_l     ;zwei Spalten weiter, Pruefung auf uebertrag nicht noetig, da er hier nie passiert
   loop_trackname 
    lda (adr2_l),y
    sta (adr1_l),y
    iny
    cpy #29       ;29 Zeichen fuer den Trackname
   bne loop_trackname
   
   ;Metadaten Parsen
        ;Datei fuer PAL oder NTSC ?
        lda (adr2_l),y    ;y ist jetzt bei #29, x noch Nummer des Tracks
        and #%10000000    ;nur Bit 7 auswerten
        cmp #%10000000
        beq NTSC
         lda #0
         jmp PAL
        NTSC
         lda #1
        PAL
         sta track_tvnorm,x 
        ;Mono oder Stereo 
        lda (adr2_l),y    ;y immer noch bei #29
        and #%01000000    ;nur Bit 6 auswerten
        cmp #%01000000
        beq Stereo
         lda #0
         jmp Mono
        Stereo
         lda #1
        Mono
         sta track_stereo,x 
        ;SID-Typ
        lda (adr2_l),y    ;y immer noch bei #29
        and #%00011100    ;nur Bit 4 bis 2 auswerten
        lsr
        lsr
        sta track_sidtyp,x    ;ist der SID-Typ in den Metadaten des Tracks FPGASID ?
        cmp #%00000011
        bne kein_fpgasid      ;wenn nein: beende die Pruefung
        lda ua_vorh           ;ist Ultimate Audio vorhanden ? Wenn nein: springe zum Setzen des Flags, dass mindestens ein Track fuer den FPGASID konvertiert wurde
        beq ua_fpga_test_ende
        lda fpgasid_vorh      ;wenn ja: prüfe, ob ein FPGASID vorhanden ist
        bne ua_fpga_test_ende ;wenn FPGASID vorhanden: springe zum Setzen des Flags, dass mindestens ein Track fuer den FPGASID konvertiert wurde
        lda #7
        sta track_sidtyp,x    ;wenn FPGASID nicht vorhanden, aber Ultimat Audio vorhanden: aendere den SID-Typ auf 7 (Ultimate Audio)
        jmp kein_fpgasid
        ua_fpga_test_ende
        lda #1
        sta fpgasid_datei     ;mindestens ein Track wurde fuer den FPGASID konvertiert
        kein_fpgasid
        ;Format
        lda (adr2_l),y    ;y immer noch bei #29
        and #%00000011    ;nur Bit 1 und 0 ausweren
        sta track_format,x 
        
        metadaten_neu_parsen_1
        lda ua_reparse    
        beq metadaten_neu_parsen_2
        ldy #29              ;Neuparsen der Abspielparameter nach Umschalten zwischen Ultimate Audio und FPGASID
        metadaten_neu_parsen_2

        ;Multiplikator fuer UA auf 1 (wird bei Bedarf geaendert und beim Spulen auch fuer Nicht-UA-Tracks verwendet)
        lda #1
        sta ua_track_mult,x
        ;Zusatzinformationen (zu verwendende DigiMax-Adresse, die Abfrage der Mahoney-Tabelle wird in dieser Version nicht durchgefuehrt: diese Metadaten sind vorbereitet fuer eventuelle zukuenftige Erweiterungen)
        lda track_sidtyp,x     
        cmp #4                ;nur bei DigiMax-Track ausfuehren
        bne zusatzinfos_ende
        lda #$DD              ;User-Port (Standardwert)   
        sta digimax_adr_h
        lda #$01                 
        sta digimax_adr_l
        lda (adr2_l),y
        and #%11000000
        beq zusatzinfos_ende
        lda #$DE              ;Cartridge   
        sta digimax_adr_h
        lda #$00                 
        sta digimax_adr_l
        zusatzinfos_ende
        ;Bits pro Sample
        ldy #30
        lda (adr2_l),y
        and #%00011111
        sta track_bits,x
        cmp #16                ;bei 16 Bit liegt Ultimate Audio High Quality vor (Flag blockiert Umschaltung auf FPGASID)
        bne kleiner_16_bits
        lda #1
        sta ua_hq
        kleiner_16_bits
        ;Samplerate
        samplerate_erneut_parsen
        ldy #31
        lda (adr2_l),y
        sta track_srate,x
        ;track_sr_h auf 0 (wird nur bei Bedarf geaendert)
        ldy #0
        sty track_sr_h,x
        
        ;Takte zwischen den Samples ermitteln anhand Lookuptable
        jsr tvnorm_init_track
        lda track_tvnorm,x
        lda track_srate,x
        cmp lup_srate
        beq srate0
        cmp lup_srate+1
        beq srate1a
        cmp lup_srate+2
        beq srate2a
        cmp lup_srate+3
        beq srate3a
        cmp #$22 ;(nur Ultimate Audio)
        beq srate4a
        cmp #$32 ;(nur Ultimate Audio)
        beq srate5a
        cmp #$44 ;(nur Ultimate Audio)
        beq srate6a
        cmp #$48 ;(nur Ultimate Audio)
        beq srate7a
        jmp Fehler_in_der_Datei  ;wenn keine gueltige Samplerate im Header steht
        srate1a
        jmp srate1
        srate2a
        jmp srate2
        srate3a
        jmp srate3
        srate4a
        jmp srate4
        srate5a
        jmp srate5
        srate6a
        jmp srate6
        srate7a
        jmp srate7               ;Umgehung des Sprunglimits fuer beq
        
        ;Hilfsdatentabelle fuer Schnellzugriff erstellen (Takte zw. Samples, Samplerate fuer Anzeige, Samplerate fuer Spielzeitberechnung)
        srate0
        lda #$31          ;15 kHz
        sta track_srate_z,x
        lda #$35
        sta track_srate_e,x
        lda track_stereo,x
        bne stereo0
        lda lup_sr_m
        sta track_sr_m,x
        lda lup_sr_l
        sta track_sr_l,x
        lda track_sidtyp,x       ;pruefe, ob Track Ultimate Audio, wenn ja: springe zu den noetigen Einstellungen
        cmp #7
        beq srate0_ua
        lda lup_takte
        sta track_takte,x
        jmp srate_ok
         stereo0
         lda #1                  ;Flag fuer Ultimate Audio High Quality
         sta ua_hq
         lda lup_sr_st_m
         sta track_sr_m,x
         lda lup_sr_st_l
         sta track_sr_l,x
         
        srate0_ua
        lda #2
        sta ua_track_mult,x
        lda tvnorm_c64
        bne srate0_ua_NTSC
        lda track_tvnorm,x
        bne srate0_ua_Datei_NTSC
        lda #126
        sta track_takte,x  
        lda #$01
        sta ua_track_T_h,x
        lda #$8F
        sta ua_track_T_l,x
        jmp srate_ok
        srate0_ua_Datei_NTSC
        lda #125
        sta track_takte,x  
        lda #$01
        sta ua_track_T_h,x
        lda #$8D
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate0_ua_NTSC
        lda track_tvnorm,x
        beq srate0_ua_NTSC_Datei_PAL
        lda #130
        sta track_takte,x  
        lda #$01
        sta ua_track_T_h,x
        lda #$8D
        sta ua_track_T_l,x
        jmp srate_ok
        srate0_ua_NTSC_Datei_PAL
        lda #130
        sta track_takte,x  
        lda #$01
        sta ua_track_T_h,x
        lda #$8F
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate1
        lda #$31          ;11 kHz
        sta track_srate_z,x
        sta track_srate_e,x
        lda track_stereo,x
        bne stereo1
        lda lup_sr_m+1
        sta track_sr_m,x
        lda lup_sr_l+1
        sta track_sr_l,x
        lda track_sidtyp,x        ;pruefe, ob Track Ultimate Audio, wenn ja: springe zu den noetigen Einstellungen
        cmp #7
        beq srate1_ua        
        lda lup_takte+1
        sta track_takte,x
        jmp srate_ok
         stereo1
         lda lup_sr_st_m+1
         sta track_sr_m,x
         lda lup_sr_st_l+1
         sta track_sr_l,x
         lda track_sidtyp,x       ;pruefe, ob Track Ultimate Audio, wenn ja: springe zu den noetigen Einstellungen
         cmp #7
         beq srate1_ua       
         lda lup_takte+1
         sta track_takte,x
         jmp srate_ok

        srate1_ua
        lda #2
        sta ua_track_mult,x
        lda tvnorm_c64
        bne srate1_ua_NTSC
        lda track_tvnorm,x
        bne srate1_ua_Datei_NTSC
        lda #168
        sta track_takte,x  
        lda #$02
        sta ua_track_T_h,x
        lda #$14
        sta ua_track_T_l,x
        jmp srate_ok
        srate1_ua_Datei_NTSC
        lda #167
        sta track_takte,x  
        lda #$02
        sta ua_track_T_h,x
        lda #$11
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate1_ua_NTSC
        lda track_tvnorm,x
        beq srate1_ua_NTSC_Datei_PAL
        lda #173
        sta track_takte,x  
        lda #$02
        sta ua_track_T_h,x
        lda #$11
        sta ua_track_T_l,x
        jmp srate_ok
        srate1_ua_NTSC_Datei_PAL
        lda #174
        sta track_takte,x  
        lda #$02
        sta ua_track_T_h,x
        lda #$14
        sta ua_track_T_l,x
        jmp srate_ok
         
        srate2
        lda #$20          ;7 kHz
        sta track_srate_z,x
        lda #$37
        sta track_srate_e,x
        lda track_stereo,x
        bne stereo2
        lda lup_sr_m+2
        sta track_sr_m,x
        lda lup_sr_l+2
        sta track_sr_l,x
        lda track_sidtyp,x        ;pruefe, ob Track Ultimate Audio, wenn ja: springe zu den noetigen Einstellungen
        cmp #7
        beq srate2_ua        
        lda lup_takte+2
        sta track_takte,x
        lda track_bits,x
        cmp #8
        bne vierbit2
        jmp srate_ok
         stereo2
         lda lup_sr_st_m+2
         sta track_sr_m,x
         lda lup_sr_st_l+2
         sta track_sr_l,x
         lda track_sidtyp,x       ;pruefe, ob Track Ultimate Audio, wenn ja: springe zu den noetigen Einstellungen
         cmp #7
         beq srate2_ua       
         lda lup_takte+2
         sta track_takte,x
         jmp srate_ok
         vierbit2
         lda lup_sr_4b_h+2
         sta track_sr_m,x
         lda lup_sr_4b_l+2
         sta track_sr_l,x
         jmp srate_ok      

        srate2_ua
        lda #1
        sta ua_track_mult,x
        lda tvnorm_c64
        bne srate2_ua_NTSC
        lda track_tvnorm,x
        bne srate2_ua_Datei_NTSC
        lda #125
        sta track_takte,x  
        lda #$03
        sta ua_track_T_h,x
        lda #$1F
        sta ua_track_T_l,x
        jmp srate_ok
        srate2_ua_Datei_NTSC
        lda #125
        sta track_takte,x  
        lda #$03
        sta ua_track_T_h,x
        lda #$1A
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate2_ua_NTSC
        lda track_tvnorm,x
        beq srate2_ua_NTSC_Datei_PAL
        lda #129
        sta track_takte,x  
        lda #$03
        sta ua_track_T_h,x
        lda #$1A
        sta ua_track_T_l,x
        jmp srate_ok
        srate2_ua_NTSC_Datei_PAL
        lda #130
        sta track_takte,x  
        lda #$03
        sta ua_track_T_h,x
        lda #$1F
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate3
        lda #$20          ;5 kHz
        sta track_srate_z,x
        lda #$35
        sta track_srate_e,x
        lda track_stereo,x
        bne stereo3
        lda lup_sr_m+3
        sta track_sr_m,x
        lda lup_sr_l+3
        sta track_sr_l,x
        lda track_sidtyp,x       ;pruefe, ob Track Ultimate Audio, wenn ja: springe zu den noetigen Einstellungen
        cmp #7
        beq srate3_ua        
        lda lup_takte+3
        sta track_takte,x
        lda track_bits,x
        cmp #8
        bne vierbit3
        jmp srate_ok
         stereo3
         lda lup_sr_st_m+3
         sta track_sr_m,x
         lda lup_sr_st_l+3
         sta track_sr_l,x
         lda track_sidtyp,x       ;pruefe, ob Track Ultimate Audio, wenn ja: springe zu den noetigen Einstellungen
         cmp #7
         beq srate3_ua       
         lda lup_takte+3
         sta track_takte,x
         jmp srate_ok
         vierbit3
         lda lup_sr_4b_h+3
         sta track_sr_m,x
         lda lup_sr_4b_l+3
         sta track_sr_l,x
         jmp srate_ok  
         
        srate3_ua
        lda #1
        sta ua_track_mult,x
        lda tvnorm_c64
        bne srate3_ua_NTSC
        lda track_tvnorm,x
        bne srate3_ua_Datei_NTSC
        lda #167
        sta track_takte,x  
        lda #$04
        sta ua_track_T_h,x
        lda #$29
        sta ua_track_T_l,x
        jmp srate_ok
        srate3_ua_Datei_NTSC
        lda #166
        sta track_takte,x  
        lda #$04
        sta ua_track_T_h,x
        lda #$23
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate3_ua_NTSC
        lda track_tvnorm,x
        beq srate3_ua_NTSC_Datei_PAL
        lda #173
        sta track_takte,x  
        lda #$04
        sta ua_track_T_h,x
        lda #$23
        sta ua_track_T_l,x
        jmp srate_ok
        srate3_ua_NTSC_Datei_PAL
        lda #174
        sta track_takte,x  
        lda #$04
        sta ua_track_T_h,x
        lda #$29
        sta ua_track_T_l,x
        jmp srate_ok

        srate4
        lda #$32                ;22 kHz (22050 Hz), nur fuer Ultimate Audio
        sta track_srate_z,x
        sta track_srate_e,x
        lda #1                  ;Flag fuer Ultimate Audio High Quality
        sta ua_hq        
        lda track_stereo,x
        bne stereo4
        lda #$56
        sta track_sr_m,x
        lda #$22
        sta track_sr_l,x
        jmp srate4_ua
         stereo4
         lda #$AC
         sta track_sr_m,x
         lda #$44
         sta track_sr_l,x
        
        srate4_ua
        lda #3
        sta ua_track_mult,x
        lda tvnorm_c64
        bne srate4_ua_NTSC
        lda #133
        sta track_takte,x  
        lda #$01
        sta ua_track_T_h,x
        lda #$19
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate4_ua_NTSC
        lda #138
        sta track_takte,x  
        lda #$01
        sta ua_track_T_h,x
        lda #$19
        sta ua_track_T_l,x
        jmp srate_ok

        srate5
        lda #$33          ;32 kHz, nur fuer Ultimate Audio
        sta track_srate_z,x
        lda #$32
        sta track_srate_e,x
        lda #1                  ;Flag fuer Ultimate Audio High Quality
        sta ua_hq        
        lda track_stereo,x
        bne stereo5
        lda #$7D
        sta track_sr_m,x
        lda #$00
        sta track_sr_l,x
        jmp srate5_ua
         stereo5
         lda #$FA
         sta track_sr_m,x
         lda #$00
         sta track_sr_l,x
        
        srate5_ua
        lda #4
        sta ua_track_mult,x
        lda tvnorm_c64
        bne srate5_ua_NTSC
        lda #122
        sta track_takte,x  
        lda #$00
        sta ua_track_T_h,x
        lda #$C1
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate5_ua_NTSC
        lda #127
        sta track_takte,x  
        lda #$00
        sta ua_track_T_h,x
        lda #$C1
        sta ua_track_T_l,x
        jmp srate_ok

        srate6
        lda #$34          ;44 kHz (44100 Hz), nur fuer Ultimate Audio
        sta track_srate_z,x
        sta track_srate_e,x
        lda #1                  ;Flag fuer Ultimate Audio High Quality
        sta ua_hq        
        lda track_stereo,x
        bne stereo6
        lda #$AC
        sta track_sr_m,x
        lda #$44
        sta track_sr_l,x
        jmp srate6_ua
         stereo6
         lda #$01
         sta track_sr_h,x
         lda #$58
         sta track_sr_m,x
         lda #$88
         sta track_sr_l,x
        
        srate6_ua
        lda #6
        sta ua_track_mult,x
        lda tvnorm_c64
        bne srate6_ua_NTSC
        lda #134
        sta track_takte,x  
        lda #$00
        sta ua_track_T_h,x
        lda #$8C
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate6_ua_NTSC
        lda #138
        sta track_takte,x  
        lda #$00
        sta ua_track_T_h,x
        lda #$8C
        sta ua_track_T_l,x
        jmp srate_ok

        srate7
        lda #$34          ;48 kHz, nur fuer Ultimate Audio
        sta track_srate_z,x
        lda #$38
        sta track_srate_e,x
        lda #1                  ;Flag fuer Ultimate Audio High Quality
        sta ua_hq        
        lda track_stereo,x
        bne stereo7
        lda #$BB
        sta track_sr_m,x
        lda #$80
        sta track_sr_l,x
        jmp srate7_ua
         stereo7
         lda #$01
         sta track_sr_h,x
         lda #$77
         sta track_sr_m,x
         lda #$00
         sta track_sr_l,x
         
        srate7_ua
        lda #6
        sta ua_track_mult,x
        lda tvnorm_c64
        bne srate7_ua_NTSC
        lda #122
        sta track_takte,x  
        lda #$00
        sta ua_track_T_h,x
        lda #$80
        sta ua_track_T_l,x
        jmp srate_ok
        
        srate7_ua_NTSC
        lda #127
        sta track_takte,x  
        lda #$00
        sta ua_track_T_h,x
        lda #$80
        sta ua_track_T_l,x
        jmp srate_ok
        srate_ok
        
        ;fuer UA-Tracks: wenn Ultimate Audio nicht vorhanden, aber FPGASID vorhanden: aendere den SID-Typ auf FPGASID, sofern das Format unterstuetzt wird
        lda track_sidtyp,x
        cmp #7
        bne ua_zu_fpga_ende  ;Pruefung nur fuer UA-Tracks durchfuehren, sonst ueberspringen
        lda ua_vorh
        bne ua_zu_fpga_ende  ;Pruefung beenden, wenn UA vorhanden
        lda fpgasid_vorh
        beq ua_zu_fpga_ende  ;Pruefung beenden, wenn FPGASID nicht vorhanden
        lda ua_hq        
        bne ua_zu_fpga_ende  ;Pruefung beenden, wenn Tracks mit Ultimate Audio High Quality in der reu-Datei vorhanden (verursacht Fehlermeldung, wenn UA nicht aktiviert)     
        lda #3               ;wenn unterstuetzt: aendere Format auf 3 (FPGASID)
        sta track_sidtyp,x
        jmp samplerate_erneut_parsen ;zwecks Korrektur der Trackparameter (insb. Anzahl der Takte zwischen den Samples)
        ua_zu_fpga_ende
        
        ;fuer UA-Tracks: wenn Ultimate Audio und FPGASID nicht vorhanden, oder FPGASID vorhanden, aber UA-Track hat nicht unterstuetzes Format: Fehlermeldung generieren
        lda track_sidtyp,x
        cmp #7
        bne ua_nicht_vorh_kein_Fehler
        lda ua_vorh
        bne ua_nicht_vorh_kein_Fehler
        jmp Fehler_UA_nicht_vorh
        ua_nicht_vorh_kein_Fehler

        ;bei Ultimate Audio Stereo den Multiplikator verdoppeln (zwecks korrekter Spielzeitanzeige)
        lda track_stereo,x
        beq srate_ok_2
        asl ua_track_mult,x
        srate_ok_2
        ;bei Ultimate Audio 16 Bit den Multiplikator verdoppeln (zwecks korrekter Spielzeitanzeige) und rechnerische Samplerate verdoppeln (zwecks korrekter Berechnung der Trackspielzeit)
        lda track_bits,x
        cmp #16
        bne srate_ok_3
        asl ua_track_mult,x
        clc
        rol track_sr_l,x
        rol track_sr_m,x
        rol track_sr_h,x
        srate_ok_3
        
        ;Multiplikatoren zum Spulen berechnen (damit UA-Tracks mit hohen Datenraten und Stereotracks schneller spulen)
        lda ua_track_mult,x
        ldy track_stereo,x
        beq spulen_multi_1
        asl      ;bei Stereo Multiplikator x2
        spulen_multi_1
        sta temp   
        ;Spulfaktoren vorberechnen (Faktor 8 fürs Spulen, Faktor 10 fuer die Reserve)
        asl      ;Faktor 8 durch 3x Bitverschiebung nach Links
        asl
        asl
        sta ua_spulen_f2,x
        clc
        adc temp ;Faktor 10, wenn man jetzt den Wert noch 2x addiert
        adc temp
        sta ua_spulen_f1,x
        
        ;entscheiden, welcher Player fuer den Track der richtige ist
        lda track_sidtyp,x      ;pruefen, ob Ultimate Audio - Track
        cmp #7
        beq playerwahl_ua
        lda track_stereo,x
        beq playerwahl1
        lda #>player_stereo
        sta track_player_h,x
        lda #<player_stereo
        sta track_player_l,x
        jmp playerwahl_fertig
        playerwahl1
         lda track_bits,x
         cmp #8
         bne playerwahl2
         lda #>player_mono_8bit
         sta track_player_h,x
         lda #<player_mono_8bit
         sta track_player_l,x   
         jmp playerwahl_fertig     
        playerwahl2 
         lda #>player_mono_4bit
         sta track_player_h,x
         lda #<player_mono_4bit
         sta track_player_l,x   
         jmp playerwahl_fertig     
        playerwahl_ua
         lda #>player_ultimate_audio
         sta track_player_h,x
         lda #<player_ultimate_audio
         sta track_player_l,x
        playerwahl_fertig 
        ;entscheiden, welche Initialisierung fuer den Track die richtige ist
        lda track_sidtyp,x
        cmp #%00000111             ;Ultimate Audio
        beq initwahl_ua            
        cmp #%00000011             ;FPGASID only
        bne initwahl_2
        lda track_stereo,x
        bne initwahl_1
        lda #>init_fpgasid_mono    ;FPGA mono
        sta track_init_h,x
        lda #<init_fpgasid_mono
        sta track_init_l,x
        jmp initwahl_fertig
        initwahl_1
        lda #>init_fpgasid_stereo  ;FPGA Stereo
        sta track_init_h,x
        lda #<init_fpgasid_stereo
        sta track_init_l,x
        jmp initwahl_fertig        
        initwahl_2                 ;Standard-SID
         lda track_bits,x
         cmp #8
         bne initwahl_4
         lda track_stereo,x 
         bne initwahl_3
         lda #>init_mono_8bit      ;Standard-SID Mono 8 Bit
         sta track_init_h,x
         lda #<init_mono_8bit
         sta track_init_l,x
         jmp initwahl_fertig
        initwahl_3
         lda #>init_stereo_8bit    ;Standard-SID Stereo 8 Bit
         sta track_init_h,x
         lda #<init_stereo_8bit
         sta track_init_l,x
         jmp initwahl_fertig
        initwahl_4                 ;Standard-SID Mono 4 Bit
         lda #>init_mono_4bit
         sta track_init_h,x
         lda #<init_mono_4bit
         sta track_init_l,x
         jmp initwahl_fertig
        initwahl_ua                 ;Ultimate Audio
         lda track_stereo,x
         bne initwahl_ua_stereo
         lda #>init_ultimate_audio_mono
         sta track_init_h,x
         lda #<init_ultimate_audio_mono
         sta track_init_l,x          
         jmp initwahl_fertig
        initwahl_ua_stereo
         lda #>init_ultimate_audio_stereo
         sta track_init_h,x
         lda #<init_ultimate_audio_stereo
         sta track_init_l,x          
        initwahl_fertig 
        
    ;pruefen, ob TV-Norm falsch
    lda track_sidtyp,x             ;keine Pruefung bei Ultimate Audio - Tracks
    cmp #7
    beq tvnorm_pruefung_ende
    
    lda tvnorm_falsch              ;Vermeidung der Wiederholung der Fehlermeldung
    bne tvnorm_pruefung_ende
    
    lda track_tvnorm,x
    cmp tvnorm_c64
    beq tvnorm_pruefung_ende
    lda track_tvnorm
    bne tvnorm_fehler_NTSC         ;Injektion von "PAL " und "NTSC" in die Fehlermeldung
    ldx #0
    loop_fehler_PAL
      lda text_PAL,x
      sta text_Fehler_TVnorm_falsch_1+33,x
      sta text_Fehler_TVnorm_falsch_6+33,x
      lda text_NTSC,x
      sta text_Fehler_TVnorm_falsch_2+16,x
      sta text_Fehler_TVnorm_falsch_7+18,x
      inx
      cpx #4
    bne loop_fehler_PAL
    jmp tvnorm_fehler_PAL
    tvnorm_fehler_NTSC
    ldx #0
    loop_fehler_NTSC
      lda text_NTSC,x
      sta text_Fehler_TVnorm_falsch_1+33,x
      sta text_Fehler_TVnorm_falsch_6+33,x
      lda text_PAL,x
      sta text_Fehler_TVnorm_falsch_2+16,x
      sta text_Fehler_TVnorm_falsch_7+18,x
      inx
      cpx #4
    bne loop_fehler_NTSC
    tvnorm_fehler_PAL
    lda tvnorm_falsch              ;verhindert erneute Fehlermeldung, wenn sie bereits bestaetigt wurde
    bne tvnorm_pruefung_ende
    lda #1
    sta tvnorm_falsch
    jmp Fehler_TVnorm_falsch
    tvnorm_pruefung_ende

   ;Adresse fuer naechsten Track berechnen
   lda adr2_l
   clc
   adc #$20
   sta adr2_l
   bcc kein_Uebertrag_2
   inc adr2_h
   kein_Uebertrag_2   
   
   lda ua_reparse
   beq metadaten_parsen_ua_fpga_umschaltung_skip
   inx
   cpx maxtrack
   beq metadaten_parsen_ua_fpga_umschaltung_ende
   jmp metadaten_neu_parsen_1
   metadaten_parsen_ua_fpga_umschaltung_ende
   lda #0
   sta ua_reparse
   ldy #253
   rts
   metadaten_parsen_ua_fpga_umschaltung_skip
        
   ;Berechnung der Tracklaenge in Sekunden (Bytes/Samplerate)
   lda tracklaenge_h,x   ;tracklaenge_h in den Dividend kopieren (mit gedrehter Endianess -> Lo-Byte zuerst)
   sta dividend+2
   lda tracklaenge_m,x
   sta dividend+1
   lda tracklaenge_l,x
   sta dividend

   lda track_sr_l,x            ;Samplerate in Divisor mit Lo-Byte zuerst
   sta divisor
   lda track_sr_m,x 
   sta divisor+1
   lda track_sr_h,x
   sta divisor+2
   
   jsr div24
   
   ldx track           
   lda dividend+1      ;Ergebnis in tracksek_h/l speichern
   sta tracksek_h,x
   lda dividend 
   sta tracksek_l,x
   
   ;Minuten und Sekunden durch Division durch 60 berechnen (die Sekunden stehen noch im Dividend) und als Screencodes speichern
   lda #60       
   sta divisor
   lda #0
   sta divisor+1
   jsr div16
   
   ldx divrest
   lda dividend     ;Minuten im Dividend (=Ergebnis)
   jsr hex2scr
   ldy track
   sta trackmin_s_e,y
   txa
   sta trackmin_s_z,y

   lda divrest     ;Sekunden im Rest
   jsr hex2scr
   ldy track
   sta tracksek_s_e,y
   txa
   sta tracksek_s_z,y
   
   ;Tracklaenge auf Bildschirm anzeigen 
   
   ldx track
   lda trackmin_s_z,x  ;Minuten Zehner
   ldy #30
   sta (adr1_l),y
     
   lda trackmin_s_e,x  ;Minuten Einer
   iny
   sta (adr1_l),y
     
   lda #$3A          ;Doppelpunkt
   iny 
   sta (adr1_l),y
   
   lda tracksek_s_z,x  ;Sekunden Zehner
   iny
   sta (adr1_l),y
     
   lda tracksek_s_e,x  ;Sekunden Einer
   iny
   sta (adr1_l),y
    
   
inc track
lda track
adc #1
cmp header+$0F   ;im Header Pos $0F steht die Anzahl der REU-FS-Eintraege (bei v02 Titelanzahl+1, ab v03 $0F)
bne jmp_loop_parse_REU_FS
jmp kein_sprung  ;Workaround, da bne nur begrenzt weit springen kann
jmp_loop_parse_REU_FS
jmp loop_parse_REU_FS
kein_sprung
reserved_entry_gefunden 
lda track   ;maxtrack auf Anzahl gefundener Tracks setzen
sta maxtrack

;pruefen, ob FPGASID-Tracks in Datei, aber kein FPGASID gefunden -> wenn ja: Fehler/Warnung ausgeben
lda fpgasid_fehlt
bne fpgasid_pruefung_ende
lda fpgasid_datei
beq fpgasid_pruefung_ende
cmp fpgasid_vorh
beq fpgasid_pruefung_ende
lda #1
sta fpgasid_fehlt
jmp Fehler_kein_FPGASID
fpgasid_pruefung_ende

